@@ -1,10 +1,11 @@
 import os
 import boto3
 
-from .base import BaseOsduClient
+from ._base import BaseOsduClient
 import boto3.session
 
 
+
 class AwsOsduClient(BaseOsduClient):
     """Good for batch tasks that don't have an interactive front-end. Token management is handled 
     with the boto3 library directly through the Cognito service. You have to supply additional arguments for this.
